/* 
   Vertical Bars Pattern PAL TV Signal Generator with Arduino 
   Use this code as you want
   2007 Javier Valcarce Garcia, <javier.valcarce@gmail.com>
*/


#include <avr/io.h>
#include <util/delay.h>


/////////////////////////////////////////////////////////////////////////////////////////////////
// Pins where the 2-bit DAC is connected

#define PINA0 6 // LSB, 330 Ohm resistor
#define PINA1 7 // MSB, 1 kOhm resistor

// PINA1 PINA0 OUTPUT 
// 0     0     0.0V   - Sync level
// 1     0     0.3V   - Black level
// 0     1     0.6V   - Gray level
// 1     1     1.0V   - White level

#define LEVEL_SYNC   PORTD &= ~(1 << PINA1); PORTD &= ~(1 << PINA0);
#define LEVEL_BLACK  PORTD |=   1 << PINA1;  PORTD &= ~(1 << PINA0);
#define LEVEL_GRAY   PORTD &= ~(1 << PINA1); PORTD |=   1 << PINA0; 
#define LEVEL_WHITE  PORTD |=   1 << PINA1;  PORTD |=   1 << PINA0;

int i = 0;
/////////////////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////////////////
inline void vsync_pulse()
{
      // Sync goes low and delay 27.3 microseconds
      LEVEL_SYNC; 
      _delay_us(27);
       __asm__("nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t"); // 0.3 microseconds
       
       // Sync goes high and delay 4.7 microseconds
      LEVEL_BLACK; 
      _delay_us(4); 
      __asm__("nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t"); // 0.7 microseconds
}

/////////////////////////////////////////////////////////////////////////////////////////////////
inline void equal_pulse()
{
      // Sync pulse goes low and delay 2.3 microseconds
      LEVEL_SYNC; 
      _delay_us(2);
      __asm__("nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t");
      // Sync pulse goes high and delay 29.7 microseconds
      LEVEL_BLACK; 
      _delay_us(29);
      __asm__("nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t");
}
/////////////////////////////////////////////////////////////////////////////////////////////////
inline void hsync_pulse()
{
      // Front Porch - high for 1.65 microseconds
      LEVEL_BLACK; 
      _delay_us(1);
      __asm__("nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t");
      
      // Generate sync pulse 4.7 microseconds
     LEVEL_SYNC; 
     _delay_us(4);
     __asm__("nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t");
     
     // Back Porch 5.6 microseconds.
      LEVEL_BLACK; 
      _delay_us(5);
      __asm__("nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t""nop\n\t");
}

/////////////////////////////////////////////////////////////////////////////////////////////////
int main()
{
      register unsigned int line;
	
      /* NOTE THAT THE SIGNAL GENERATED BY THIS PROGRAM HAS A NOT VERY ACURATE TIMMING SO 
	 IT IS POSIBLE THAT THE IMAGE BLINKS ON YOUR TV SCREEN OR DOESN'T SHOW AT ALL, 
	 THIS PROGRAM WRITTEN IN C (INTEAD OF ASSEMBLER) IS ONLY A PROFF OF CONCEPT */
	
      line = 0;
      DDRD = 0xFF; // PORTD, all pins are outputs

      while(1)
      {
	    
	    // 1:
	    vsync_pulse(); vsync_pulse();
		// 2:
		vsync_pulse(); vsync_pulse();
		// 3:
		vsync_pulse(); equal_pulse();
		// 4:
		equal_pulse(); equal_pulse();
		// 5:
		equal_pulse(); equal_pulse();
		
		// 6-310 (305 lines):
		for( i = 0; i < 305; i++)
		{
			hsync_pulse();
			LEVEL_GRAY; _delay_us(8);
			LEVEL_BLACK; _delay_us(14);
			LEVEL_WHITE; _delay_us(8);
			LEVEL_BLACK; _delay_us(14);
			LEVEL_GRAY; _delay_us(8);
		}
		
		// 311:
		equal_pulse(); equal_pulse();
		// 312:
		equal_pulse(); equal_pulse();
		// 313:
	    equal_pulse(); vsync_pulse();
		// 314:
		vsync_pulse(); vsync_pulse();
		// 315:
		vsync_pulse(); vsync_pulse();
		// 316:
		equal_pulse(); equal_pulse();
		// 317:
		equal_pulse(); equal_pulse();
		
		// 318-622 (305 lines):
		for( i = 0; i < 305; i++)
		{
			hsync_pulse();
			LEVEL_GRAY; _delay_us(8);
			LEVEL_BLACK; _delay_us(14);
			LEVEL_WHITE; _delay_us(8);
			LEVEL_BLACK; _delay_us(14);
			LEVEL_GRAY; _delay_us(8);
		}
		
		// 623:
		equal_pulse(); equal_pulse();
		// 624:
		equal_pulse(); equal_pulse();
		// 625:
		equal_pulse(); equal_pulse();
		

      }
}